/*

*/
import scene.*;
import scene.models.*;
import pipeline.*;
import framebuffer.*;

import java.io.File;
import java.awt.Color;

/**
   Use this program to experiment with two-dimensional transformations.

   Try mixing the ordering of rotations, translations, and scales.
*/
public class TransformationTester
{
   public static void main(String[] args)
   {
      // Create several 2D Model objects.
      Model m1 = new GRSModel(new File("assets/grs/bronto.grs"));
      Model m2 = new GRSModel(new File("assets/grs/usa.grs"));
      Model m3 = new GRSModel(new File("assets/grs/rex.grs"));
      Model m4 = new GRSModel(new File("assets/grs/vinci.grs"));

      // Pick a model from the above models.
      Model m = m1;

      // Set the model's color.
      m.setColor(Color.black);

      // Create the Scene object that we shall render.
      Scene scene = new Scene();

      // Add the model to the Scene.
      scene.modelList.add(m);

      // Create a set of x and y axes.
      Model axes = new Axes2DModel(-1, +1, -1, +1, 24, 24);
      axes.setColor( Color.red );
      scene.modelList.add(axes);


      // Create a framebuffer to render our scene into.
      int vp_width  = 512;
      int vp_height = 512;
      FrameBuffer fb = new FrameBuffer(vp_width, vp_height);
      // Give the framebuffer a nice background color.
      fb.clearFB(java.awt.Color.lightGray);


      // Use orthographic projection
      scene.camera.perspective = false;

      for (int i = 0; i <= 36; i++)
      {
         m.model2Identity();
         // try changing the order of these transformations
         m.modelScale(0.33, 0.33, 0.33); // scale the model to fit in the view rectangle
         m.modelRotate(10*i, 0, 0, 1);
         m.modelTranslate(1, 0, 0);

         // Render again.
         fb.clearFB(java.awt.Color.lightGray);
         Pipeline.render(scene, fb);
         fb.dumpFB2File(String.format("PPM_TransformationTester_Frame%02d.ppm", i));
      }
   }
}