/*
   Rotate a square around its center, with its center at the origin.
   Demonstrate the transformation of the coordinate system.
*/
import scene.*;
import scene.models.*;
import pipeline.*;
import framebuffer.*;

import java.awt.Color;

/**
   In the last example file, we said that this code

      square.model2Identity();
      square.modelRotate(6*i, 0,0,1);
      square.modelTranslate(-0.5, -0.5, 0);

   when read from the top down, should be read as a sequence of
   transformations of the coordinate system. (When it is read from
   the bottom up, it should be read as a sequence of transformations
   of vertices.) This program draws a visualization of the coordinate
   system being transformed.

   In this animation, each frame of the last example animation is turned
   into four frames  The first frame represents this line of code

      square.model2Identity();

   which restores the coordinates system to the original one. The second
   frame represents this line of code

      square.modelRotate(6*i, 0,0,1);

   which rotates the coordinate system. The third frame represents this
   line of code

      square.modelTranslate(-0.5, -0.5, 0);

   which translates the rotated coordinate system. The last frame then
   draws the square model in the last coordinate system.
*/
public class Example_02b
{
   public static void main(String[] args)
   {
      // Create the Scene object that we shall render.
      Scene scene = new Scene();

      // Create a set of x and y axes.
      Model axes = new Axes2DModel(-2, +2, -2, +2, 8, 8);
      axes.setColor( Color.red );
      // Push the model away from where the camera is.
      axes.modelTranslate(0, 0, -2);
      scene.addModel(axes);

      // Create two shorter sets of x and y axes.
      Model sAxes1 = new Axes2DModel(-0.5, +0.5, -0.5, +0.5, 2, 2);
      Model sAxes2 = new Axes2DModel(-0.5, +0.5, -0.5, +0.5, 2, 2);
      sAxes1.setColor( new Color(150, 0, 0) );
      sAxes2.setColor( new Color(100, 0, 0) );
      scene.addModel(sAxes1, sAxes2);

      // Create a FrameBuffer to render our scene into.
      int width  = 512;
      int height = 512;
      FrameBuffer fb = new FrameBuffer(width, height);


      Model square = new Model();
      square.addLineSegment(new LineSegment(new Vertex(0, 0, 0), new Vertex(1, 0, 0)));
      square.addLineSegment(new LineSegment(new Vertex(1, 0, 0), new Vertex(1, 1, 0)));
      square.addLineSegment(new LineSegment(new Vertex(1, 1, 0), new Vertex(0, 1, 0)));
      square.addLineSegment(new LineSegment(new Vertex(0, 1, 0), new Vertex(0, 0, 0)));
      // Give the model a random color.
      square.setColorRandom();
      // Add the square to the Scene.
      scene.addModel( square );


      // Rotate the square around its center, with its center at the origin.
      for (int i = 0; i <= 60; i++)
      {
         sAxes1.model2Identity();
         sAxes2.model2Identity();
         square.model2Identity();
         // Push the models away from where the camera is.
         sAxes1.modelTranslate(0, 0, -2);
         sAxes2.modelTranslate(0, 0, -2);
         square.modelTranslate(0, 0, -2);

         sAxes1.modelRotate(6*i, 0,0,1);

         sAxes2.modelRotate(6*i, 0,0,1);
         sAxes2.modelTranslate(-0.5, -0.5, 0);

         square.modelRotate(6*i, 0,0,1);
         square.modelTranslate(-0.5, -0.5, 0);

         sAxes1.hidden = true;
         sAxes2.hidden = true;
         square.hidden = true;
         fb.clearFB(Color.black);
         Pipeline.render(scene, fb);
         fb.dumpFB2File(String.format("PPM_Example_Frame%03da.ppm", i));

         sAxes1.hidden = false;
         fb.clearFB(Color.black);
         Pipeline.render(scene, fb);
         fb.dumpFB2File(String.format("PPM_Example_Frame%03db.ppm", i));

         sAxes2.hidden = false;
         fb.clearFB(Color.black);
         Pipeline.render(scene, fb);
         fb.dumpFB2File(String.format("PPM_Example_Frame%03dc.ppm", i));

         square.hidden = false;
         fb.clearFB(Color.black);
         Pipeline.render(scene, fb);
         fb.dumpFB2File(String.format("PPM_Example_Frame%03dd.ppm", i));
      }
   }
}
