/*
   Rotate a square around its center, with its center at the origin.
   Do this by "accumulating" rotations.
*/
import scene.*;
import scene.models.*;
import pipeline.*;
import framebuffer.*;

import java.awt.Color;

/**
   The last animation showed how the coordinate system gets transformed
   by the code. Notice in that animation that we always "go back" to the
   original coordinate system and then rotate it. This example shows
   we don't need to go back to the original coordinate system. We can "save"
   the rotated coordinate system, translate it, then draw, then "return" to
   the rotated coordinate system, save it again, translate it again, then
   draw, etc.

   This is not a better way to create the animation since it is harder to
   read. But sometimes this idea of "saving" and "restoring" a coordinates
   system can be useful.
*/
public class Example_02c
{
   public static void main(String[] args)
   {
      // Create the Scene object that we shall render.
      Scene scene = new Scene();

      // Create a set of x and y axes.
      Model axes = new Axes2DModel(-2, +2, -2, +2, 8, 8);
      axes.setColor( Color.red );
      // Push the model away from where the camera is.
      axes.modelTranslate(0, 0, -2);
      scene.addModel(axes);


      // Create a FrameBuffer to render our scene into.
      int width  = 512;
      int height = 512;
      FrameBuffer fb = new FrameBuffer(width, height);


      Model square = new Model();
      square.addLineSegment(new LineSegment(new Vertex(0, 0, 0), new Vertex(1, 0, 0)));
      square.addLineSegment(new LineSegment(new Vertex(1, 0, 0), new Vertex(1, 1, 0)));
      square.addLineSegment(new LineSegment(new Vertex(1, 1, 0), new Vertex(0, 1, 0)));
      square.addLineSegment(new LineSegment(new Vertex(0, 1, 0), new Vertex(0, 0, 0)));
      // Give the model a random color.
      square.setColorRandom();
      // Add the square to the Scene.
      scene.addModel( square );

      // Push the square away from where the camera is.
      square.modelTranslate(0, 0, -2);

      // Rotate the square around its center, with its center at the origin.
      Matrix save = square.modelMatrix;  // save the coordinate system
      for (int i = 0; i <= 60; i++)
      {
         square.modelTranslate(-0.5, -0.5, 0); // update the coordinate system

         // Render again.
         fb.clearFB(Color.black);
         Pipeline.render(scene, fb);
         fb.dumpFB2File(String.format("PPM_Example_Frame%03d.ppm", i));

         square.modelMatrix = save;    // restore the coordinate system
         square.modelRotate(6, 0,0,1); // accumulate a rotation
         save = square.modelMatrix;    // save the coordinate system again
      }
   }
}
