/*
   Rotate a square around its center, with its center at the origin.
   Do this by "accumulating" rotations.
   Demonstrate the transformation of the coordinate system.
*/
import scene.*;
import scene.models.*;
import pipeline.*;
import framebuffer.*;

import java.awt.Color;

/**
   Rotate a square around its center, with its center at the origin.
   Do this by "accumulating" rotations.
   Demonstrate the transformation of the coordinate system.
*/
public class Example_02d
{
   public static void main(String[] args)
   {
      // Create the Scene object that we shall render.
      Scene scene = new Scene();

      // Create a set of x and y axes.
      Model axes = new Axes2DModel(-2, +2, -2, +2, 8, 8);
      axes.setColor( Color.red );
      // Push the model away from where the camera is.
      axes.modelTranslate(0, 0, -2);
      scene.addModel(axes);

      // Create two shorter sets of x and y axes.
      Model sAxes1 = new Axes2DModel(-0.5, +0.5, -0.5, +0.5, 2, 2);
      Model sAxes2 = new Axes2DModel(-0.5, +0.5, -0.5, +0.5, 2, 2);
      sAxes1.setColor( new Color(150, 0, 0) );
      sAxes2.setColor( new Color(100, 0, 0) );
      scene.addModel(sAxes1, sAxes2);

      // Create a FrameBuffer to render our scene into.
      int width  = 512;
      int height = 512;
      FrameBuffer fb = new FrameBuffer(width, height);


      Model square = new Model();
      square.addLineSegment(new LineSegment(new Vertex(0, 0, 0), new Vertex(1, 0, 0)));
      square.addLineSegment(new LineSegment(new Vertex(1, 0, 0), new Vertex(1, 1, 0)));
      square.addLineSegment(new LineSegment(new Vertex(1, 1, 0), new Vertex(0, 1, 0)));
      square.addLineSegment(new LineSegment(new Vertex(0, 1, 0), new Vertex(0, 0, 0)));
      // Give the model a random color.
      square.setColorRandom();
      // Add the square to the Scene.
      scene.addModel( square );

      // Push the models away from where the camera is.
      square.modelTranslate(0, 0, -2);
      sAxes1.modelTranslate(0, 0, -2);
      sAxes2.modelTranslate(0, 0, -2);

      // Rotate the square around its center, with its center at the origin.
      Matrix save  = square.modelMatrix;  // save the coordinate system
      Matrix save1 = sAxes1.modelMatrix;
      Matrix save2 = sAxes2.modelMatrix;
      for (int i = 0; i <= 60; i++)
      {
         sAxes2.modelTranslate(-0.5, -0.5, 0);
         square.modelTranslate(-0.5, -0.5, 0); // update the coordinate system

         sAxes1.hidden = false;
         sAxes2.hidden = false;
         square.hidden = true;
         fb.clearFB(Color.black);
         Pipeline.render(scene, fb);
         fb.dumpFB2File(String.format("PPM_Example_Frame%03da.ppm", i));

         square.hidden = false;
         fb.clearFB(Color.black);
         Pipeline.render(scene, fb);
         fb.dumpFB2File(String.format("PPM_Example_Frame%03db.ppm", i));

         sAxes1.modelMatrix = save1;    // restore the coordinate system
         sAxes2.modelMatrix = save2;
         square.modelMatrix = save;

         sAxes1.hidden = false;
         sAxes2.hidden = true;
         square.hidden = true;
         fb.clearFB(Color.black);
         Pipeline.render(scene, fb);
         fb.dumpFB2File(String.format("PPM_Example_Frame%03dc.ppm", i));

         sAxes1.modelRotate(6, 0,0,1);  // accumulate a rotation
         sAxes2.modelRotate(6, 0,0,1);
         square.modelRotate(6, 0,0,1);

         fb.clearFB(Color.black);
         Pipeline.render(scene, fb);
         fb.dumpFB2File(String.format("PPM_Example_Frame%03dd.ppm", i));

         save1 = sAxes1.modelMatrix;    // save the coordinate system again
         save2 = sAxes2.modelMatrix;
         save  = square.modelMatrix;
      }
   }
}
