/*

*/
import scene.*;
import scene.models.*;
import pipeline.*;
import framebuffer.*;

import java.awt.Color;

/**
   Rotate the unit square around the point (3/4, 3/4) in the
   square's model coordinate system, but with the center of
   rotation at the point (1/4, 1/4) in the.camera coordinate
   system.

   Also demonstrate the transformation of the coordinate system.
*/
public class Example_04b
{
   public static void main(String[] args)
   {
      // Create the Scene object that we shall render.
      Scene scene = new Scene();

      // Create a set of x and y axes.
      Model axes = new Axes2DModel(-2, +2, -2, +2, 8, 8);
      axes.setColor( Color.red );
      // Push the model away from where the camera is.
      axes.modelTranslate(0, 0, -2);
      scene.addModel(axes);

      // Create three shorter sets of x and y axes.
      Model sAxes1 = new Axes2DModel(-0.5, +0.5, -0.5, +0.5, 2, 2);
      Model sAxes2 = new Axes2DModel(-0.5, +0.5, -0.5, +0.5, 2, 2);
      Model sAxes3 = new Axes2DModel(-0.5, +0.5, -0.5, +0.5, 2, 2);
      sAxes1.setColor( new Color(150, 0, 0) );
      sAxes2.setColor( new Color(100, 0, 0) );
      sAxes3.setColor( new Color( 50, 0, 0) );
      scene.addModel(sAxes1, sAxes2, sAxes3);

      // Create a FrameBuffer to render our scene into.
      int width  = 512;
      int height = 512;
      FrameBuffer fb = new FrameBuffer(width, height);


      Model square = new Model();
      square.addLineSegment(new LineSegment(new Vertex(0, 0, 0), new Vertex(1, 0, 0)));
      square.addLineSegment(new LineSegment(new Vertex(1, 0, 0), new Vertex(1, 1, 0)));
      square.addLineSegment(new LineSegment(new Vertex(1, 1, 0), new Vertex(0, 1, 0)));
      square.addLineSegment(new LineSegment(new Vertex(0, 1, 0), new Vertex(0, 0, 0)));
      // Give the model a random color.
      square.setColorRandom();
      // Add the square to the Scene.
      scene.addModel( square );


      // Rotate a square around its upper right hand corner.
      for (int i = 0; i <= 60; i++)
      {
         sAxes1.model2Identity();
         sAxes2.model2Identity();
         sAxes3.model2Identity();
         square.model2Identity();
         // Push the models away from where the camera is.
         sAxes1.modelTranslate(0, 0, -2);
         sAxes2.modelTranslate(0, 0, -2);
         sAxes3.modelTranslate(0, 0, -2);
         square.modelTranslate(0, 0, -2);

         sAxes1.modelTranslate(0.25, 0.25, 0);

         sAxes2.modelTranslate(0.25, 0.25, 0);
         sAxes2.modelRotate(6*i, 0,0,1);

         sAxes3.modelTranslate(0.25, 0.25, 0);
         sAxes3.modelRotate(6*i, 0,0,1);
         sAxes3.modelTranslate(-0.75, -0.75, 0);

         square.modelTranslate(0.25, 0.25, 0);
         square.modelRotate(6*i, 0,0,1);
         square.modelTranslate(-0.75, -0.75, 0);

         sAxes1.hidden = true;
         sAxes2.hidden = true;
         sAxes3.hidden = true;
         square.hidden = true;
         fb.clearFB(Color.black);
         Pipeline.render(scene, fb);
         fb.dumpFB2File(String.format("PPM_Example_Frame%03da.ppm", i));

         sAxes1.hidden = false;
         fb.clearFB(Color.black);
         Pipeline.render(scene, fb);
         fb.dumpFB2File(String.format("PPM_Example_Frame%03db.ppm", i));

         sAxes2.hidden = false;
         fb.clearFB(Color.black);
         Pipeline.render(scene, fb);
         fb.dumpFB2File(String.format("PPM_Example_Frame%03dc.ppm", i));

         sAxes3.hidden = false;
         fb.clearFB(Color.black);
         Pipeline.render(scene, fb);
         fb.dumpFB2File(String.format("PPM_Example_Frame%03dd.ppm", i));

         square.hidden = false;
         fb.clearFB(Color.black);
         Pipeline.render(scene, fb);
         fb.dumpFB2File(String.format("PPM_Example_Frame%03de.ppm", i));
      }
   }
}
