/*

*/
import scene.*;
import scene.models.*;
import pipeline.*;
import framebuffer.*;

import java.awt.Color;

/**
   Rotate the letter P around the top of the P
   while scaling it twice as large for each rotation.
   The top of P should always be at the point (0, 1)..
*/
public class Example_32
{
   public static void main(String[] args)
   {
      // Create the Scene object that we shall render.
      Scene scene = new Scene();

      // Create a set of x and y axes.
      Model axes = new Axes2DModel(-4, +4, -4, +4, 16, 16);
      axes.setColor( Color.red );
      scene.modelList.add(axes);
      // Push the model away from where the camera is.
      axes.modelTranslate(0, 0, -4);

      // Add a Model to the Scene.
      P p = new P();
      scene.addModel( p );

      // Give the model a random color.
      p.setColorRandom();

      // Create a FrameBuffer to render our scene into.
      int width  = 512;
      int height = 512;
      FrameBuffer fb = new FrameBuffer(width, height);

      // Rotate the letter P around the top of the P
      // while scaling it twice as large for each rotation.
      // The top of P should always be at the point (0, 1).
      for (int i = 0; i <= 120; i++)
      {
         p.model2Identity();
         // Push the model away from where the camera is.
         p.modelTranslate(0, 0, -4);

         p.modelTranslate(0, 1, 0);
         p.modelRotate(i*6, 0,0,1);
         double scale = Math.pow(2, i/60.0);
         p.modelScale(scale, scale, 1);
         p.modelTranslate(0, -1, 0);

         // Render again.
         fb.clearFB(Color.black);
         Pipeline.render(scene, fb);
         fb.dumpFB2File(String.format("PPM_Example_Frame%03d.ppm", i));
      }

      // Why does this animation act different than the one just above?
      for (int i = 0; i <= 120; i++)
      {
         p.model2Identity();
         // Push the model away from where the camera is.
         p.modelTranslate(0, 0, -4);

         p.modelTranslate(0, 1, 0);
         p.modelRotate(i*6, 0,0,1);
         p.modelTranslate(0, -1, 0);
         double scale = Math.pow(2, i/60.0);
         p.modelScale(scale, scale, 1);

         // Render again.
         fb.clearFB(Color.black);
         Pipeline.render(scene, fb);
         fb.dumpFB2File(String.format("PPM_Example_Frame%03d.ppm", i+200));
      }

      // Why is this animation the same as the first one?
      for (int i = 0; i <= 120; i++)
      {
         p.model2Identity();
         // Push the model away from where the camera is.
         p.modelTranslate(0, 0, -4);

         p.modelTranslate(0, 1, 0);
         p.modelRotate(i*6, 0,0,1);
         double scale = Math.pow(2, i/60.0);
         p.modelTranslate(0, -scale, 0);
         p.modelScale(scale, scale, 1);

         // Render again.
         fb.clearFB(Color.black);
         Pipeline.render(scene, fb);
         fb.dumpFB2File(String.format("PPM_Example_Frame%03d.ppm", i+400));
      }

      // Try to do this animation with the transformations
      // in the following order (from top to bottom).
      // scale, translate, rotate, translate
      for (int i = 0; i <= 120; i++)
      {
         p.model2Identity();
         // Push the model away from where the camera is.
         p.modelTranslate(0, 0, -4);

         double scale = Math.pow(2, i/60.0);
         // do scale, translate, rotate, translate





         // Render again.
         fb.clearFB(Color.black);
         Pipeline.render(scene, fb);
         fb.dumpFB2File(String.format("PPM_Example_Frame%03d.ppm", i+600));
      }


   }
}
